@@ -16,13 +16,14 @@
 import configparser
 import json
 import unittest
+import os
 
 from pathlib import Path
 from unittest.mock import MagicMock
 from unittest.mock import Mock
 
 from medusa.restore_cluster import RestoreJob, expand_repeatable_option
-from medusa.config import MedusaConfig, StorageConfig, _namedtuple_from_dict
+from medusa.config import MedusaConfig, StorageConfig, _namedtuple_from_dict, CassandraConfig
 
 
 class RestoreClusterTest(unittest.TestCase):
@@ -35,12 +36,20 @@ class RestoreClusterTest(unittest.TestCase):
         config['storage'] = {
             'host_file_separator': ','
         }
+        config['cassandra'] = {
+            'config_file': os.path.join(os.path.dirname(__file__),
+                                        'resources/yaml/work/cassandra_with_tokens_and_autobootstrap.yaml'),
+            'start_cmd': '/etc/init.d/cassandra start',
+            'stop_cmd': '/etc/init.d/cassandra stop',
+            'is_ccm': '1',
+            'resolve_ip_addresses': 'False'
+        }
         self.config = MedusaConfig(
             storage=_namedtuple_from_dict(StorageConfig, config['storage']),
             monitoring={},
-            cassandra=None,
+            cassandra=_namedtuple_from_dict(CassandraConfig, config['cassandra']),
             ssh=None,
-            restore=None,
+            checks=None,
             logging=None
         )
 
@@ -69,7 +78,15 @@ class RestoreClusterTest(unittest.TestCase):
                 tokenmap = json.loads(f.read())
                 cluster_backup = MagicMock()
                 restoreJob = RestoreJob(
-                    cluster_backup, self.config, Path('/tmp'), None, "node1.mydomain.net", False, False, None
+                    cluster_backup,
+                    self.config,
+                    Path('/tmp'),
+                    None,
+                    "node1.mydomain.net",
+                    False,
+                    False,
+                    None,
+                    bypass_checks=True
                 )
 
                 target_tokenmap = json.loads(f_target.read())
@@ -89,7 +106,15 @@ class RestoreClusterTest(unittest.TestCase):
                 tokenmap = json.loads(f.read())
                 cluster_backup = MagicMock()
                 restoreJob = RestoreJob(
-                    cluster_backup, self.config, Path('/tmp'), None, "node1.mydomain.net", False, False, None
+                    cluster_backup,
+                    self.config,
+                    Path('/tmp'),
+                    None,
+                    "node1.mydomain.net",
+                    False,
+                    False,
+                    None,
+                    bypass_checks=True
                 )
 
                 target_tokenmap = json.loads(f_target.read())
@@ -144,6 +169,45 @@ class RestoreClusterTest(unittest.TestCase):
         result = expand_repeatable_option(option, sorted(list(values)))
         self.assertEqual('--keyspace k1 --keyspace k2', result)
 
+    def test_restore_is_in_place_no_diff(self):
+        node_backups = list()
+        node_backups.append(Mock())
+        with open("tests/resources/restore_cluster_tokenmap.json", 'r') as f:
+            cluster_backup = Mock()
+            tokenmap_content = f.read()
+            tokenmap = json.loads(tokenmap_content)
+            backup_tokenmap = json.loads(tokenmap_content)
+            cluster_backup.tokenmap.return_value = tokenmap
+            restoreJob = RestoreJob(cluster_backup, self.config, Path('/tmp'), None, None, False, False, None)
+            in_place = restoreJob._is_restore_in_place(tokenmap, backup_tokenmap)
+            self.assertTrue(in_place)
+
+    def test_restore_is_in_place_one_diff(self):
+        node_backups = list()
+        node_backups.append(Mock())
+        with open("tests/resources/restore_cluster_tokenmap.json", 'r') as f:
+            with open("tests/resources/restore_cluster_tokenmap.one_changed.json", 'r') as f2:
+                cluster_backup = Mock()
+                tokenmap = json.loads(f2.read())
+                backup_tokenmap = json.loads(f.read())
+                cluster_backup.tokenmap.return_value = tokenmap
+                restoreJob = RestoreJob(cluster_backup, self.config, Path('/tmp'), None, None, False, False, None)
+                in_place = restoreJob._is_restore_in_place(tokenmap, backup_tokenmap)
+                self.assertTrue(in_place)
+
+    def test_restore_is_not_in_place(self):
+        node_backups = list()
+        node_backups.append(Mock())
+        with open("tests/resources/restore_cluster_tokenmap.json", 'r') as f:
+            with open("tests/resources/restore_cluster_tokenmap.target.json", 'r') as f2:
+                cluster_backup = Mock()
+                tokenmap = json.loads(f2.read())
+                backup_tokenmap = json.loads(f.read())
+                cluster_backup.tokenmap.return_value = tokenmap
+                restoreJob = RestoreJob(cluster_backup, self.config, Path('/tmp'), None, None, False, False, None)
+                in_place = restoreJob._is_restore_in_place(tokenmap, backup_tokenmap)
+                self.assertFalse(in_place)
+
 
 if __name__ == '__main__':
     unittest.main()
